@@ -1,18 +1,29 @@
+// SPDX-FileCopyrightText: 2011 Vamsi Krishna Brahmajosyula <vamsikrishna.brahmajosyula@gmail.com>
+// SPDX-FileCopyrightText: 2011 Giovanni Campagna <gcampagna@src.gnome.org>
+// SPDX-FileCopyrightText: 2013 Debarshi Ray <debarshir@gnome.org>
+// SPDX-FileCopyrightText: 2013 Florian Müllner <fmuellner@gnome.org>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 /* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
-/* exported init enable disable */
+import Atk from 'gi://Atk';
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GMenu from 'gi://GMenu';
+import GObject from 'gi://GObject';
+import Gtk from 'gi://Gtk';
+import Meta from 'gi://Meta';
+import Shell from 'gi://Shell';
+import St from 'gi://St';
+import {EventEmitter} from 'resource:///org/gnome/shell/misc/signals.js';
 
-const {
-    Atk, Clutter, Gio, GLib, GMenu, GObject, Gtk, Meta, Shell, St,
-} = imports.gi;
-const Signals = imports.signals;
+import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
-const DND = imports.ui.dnd;
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const PanelMenu = imports.ui.panelMenu;
-const PopupMenu = imports.ui.popupMenu;
-
-const _ = ExtensionUtils.gettext;
+import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
+import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
 
 const appSys = Shell.AppSystem.get_default();
 
@@ -46,11 +57,8 @@ class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
         this.label_actor = appLabel;
 
         let textureCache = St.TextureCache.get_default();
-        let iconThemeChangedId = textureCache.connect('icon-theme-changed',
-            this._updateIcon.bind(this));
-        this.connect('destroy', () => {
-            textureCache.disconnect(iconThemeChangedId);
-        });
+        textureCache.connectObject('icon-theme-changed',
+            () => this._updateIcon(), this);
         this._updateIcon();
 
         this._delegate = this;
@@ -117,7 +125,7 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         else
             name = _('Favorites');
 
-        this.add_child(new St.Label({ text: name }));
+        this.add_child(new St.Label({text: name}));
         this.connect('motion-event', this._onMotionEvent.bind(this));
         this.connect('notify::active', this._onActiveChanged.bind(this));
     }
@@ -239,8 +247,10 @@ class ApplicationsMenu extends PopupMenu.PopupMenu {
     }
 }
 
-class DesktopTarget {
+class DesktopTarget extends EventEmitter {
     constructor() {
+        super();
+
         this._desktop = null;
         this._desktopDestroyedId = 0;
 
@@ -267,9 +277,7 @@ class DesktopTarget {
 
     _setDesktop(desktop) {
         if (this._desktop) {
-            this._desktop.disconnect(this._desktopDestroyedId);
-            this._desktopDestroyedId = 0;
-
+            this._desktop.disconnectObject(this);
             delete this._desktop._delegate;
         }
 
@@ -277,9 +285,9 @@ class DesktopTarget {
         this.emit('desktop-changed');
 
         if (this._desktop) {
-            this._desktopDestroyedId = this._desktop.connect('destroy', () => {
+            this._desktop.connectObject('destroy', () => {
                 this._setDesktop(null);
-            });
+            }, this);
             this._desktop._delegate = this;
         }
     }
@@ -319,10 +327,7 @@ class DesktopTarget {
     }
 
     destroy() {
-        if (this._windowAddedId)
-            global.window_group.disconnect(this._windowAddedId);
-        this._windowAddedId = 0;
-
+        global.window_group.disconnectObject(this);
         this._setDesktop(null);
     }
 
@@ -357,7 +362,6 @@ class DesktopTarget {
         return true;
     }
 }
-Signals.addSignalMethods(DesktopTarget.prototype);
 
 class ApplicationsButton extends PanelMenu.Button {
     static {
@@ -385,15 +389,14 @@ class ApplicationsButton extends PanelMenu.Button {
         this.name = 'panelApplications';
         this.label_actor = this._label;
 
-        this._showingId = Main.overview.connect('showing', () => {
-            this.add_accessible_state(Atk.StateType.CHECKED);
-        });
-        this._hidingId = Main.overview.connect('hiding', () => {
-            this.remove_accessible_state(Atk.StateType.CHECKED);
-        });
+        Main.overview.connectObject(
+            'showing', () => this.add_accessible_state(Atk.StateType.CHECKED),
+            'hiding', () => this.remove_accessible_state(Atk.StateType.CHECKED),
+            this);
+
         Main.wm.addKeybinding(
             'apps-menu-toggle-menu',
-            ExtensionUtils.getSettings(),
+            Extension.lookupByURL(import.meta.url).getSettings(),
             Meta.KeyBindingFlags.IGNORE_AUTOREPEAT,
             Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
             () => this.menu.toggle());
@@ -408,16 +411,16 @@ class ApplicationsButton extends PanelMenu.Button {
             });
         });
 
-        this._tree = new GMenu.Tree({ menu_basename: 'applications.menu' });
-        this._treeChangedId = this._tree.connect('changed',
-            this._onTreeChanged.bind(this));
+        this._tree = new GMenu.Tree({menu_basename: 'applications.menu'});
+        this._tree.connectObject('changed',
+            () => this._onTreeChanged(), this);
 
         this._applicationsButtons = new Map();
         this.reloadFlag = false;
         this._createLayout();
         this._display();
-        this._installedChangedId = appSys.connect('installed-changed',
-            this._onTreeChanged.bind(this));
+        appSys.connectObject('installed-changed',
+            () => this._onTreeChanged(), this);
     }
 
     _onTreeChanged() {
@@ -441,11 +444,7 @@ class ApplicationsButton extends PanelMenu.Button {
     _onDestroy() {
         super._onDestroy();
 
-        Main.overview.disconnect(this._showingId);
-        Main.overview.disconnect(this._hidingId);
-        appSys.disconnect(this._installedChangedId);
-        this._tree.disconnect(this._treeChangedId);
-        this._tree = null;
+        delete this._tree;
 
         Main.wm.removeKeybinding('apps-menu-toggle-menu');
 
@@ -509,7 +508,7 @@ class ApplicationsButton extends PanelMenu.Button {
                 }
                 let app = appSys.lookup_app(id);
                 if (!app)
-                    app = new Shell.App({ app_info: entry.get_app_info() });
+                    app = new Shell.App({app_info: entry.get_app_info()});
                 if (app.get_app_info().should_show())
                     this.applicationsByCategory[categoryId].push(app);
             } else if (nextType === GMenu.TreeItemType.SEPARATOR) {
@@ -555,8 +554,8 @@ class ApplicationsButton extends PanelMenu.Button {
     _createLayout() {
         let section = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(section);
-        this.mainBox = new St.BoxLayout({ vertical: false });
-        this.leftBox = new St.BoxLayout({ vertical: true });
+        this.mainBox = new St.BoxLayout({vertical: false});
+        this.leftBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox = new St.ScrollView({
             style_class: 'apps-menu vfade',
             x_expand: true,
@@ -578,9 +577,9 @@ class ApplicationsButton extends PanelMenu.Button {
         vscroll.connect('scroll-stop', () => (this.menu.passEvents = false));
         this.leftBox.add_child(this.categoriesScrollBox);
 
-        this.applicationsBox = new St.BoxLayout({ vertical: true });
+        this.applicationsBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox.add_actor(this.applicationsBox);
-        this.categoriesBox = new St.BoxLayout({ vertical: true });
+        this.categoriesBox = new St.BoxLayout({vertical: true});
         this.categoriesScrollBox.add_actor(this.categoriesBox);
 
         this.mainBox.add(this.leftBox);
@@ -676,22 +675,17 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 }
 
-let appsMenuButton;
+export default class AppsMenuExtension extends Extension {
+    enable() {
+        this._appsMenuButton = new ApplicationsButton();
+        const index = Main.sessionMode.panel.left.indexOf('activities') + 1;
+        Main.panel.addToStatusArea(
+            'apps-menu', this._appsMenuButton, index, 'left');
+    }
 
-/** */
-function enable() {
-    appsMenuButton = new ApplicationsButton();
-    let index = Main.sessionMode.panel.left.indexOf('activities') + 1;
-    Main.panel.addToStatusArea('apps-menu', appsMenuButton, index, 'left');
-}
-
-/** */
-function disable() {
-    Main.panel.menuManager.removeMenu(appsMenuButton.menu);
-    appsMenuButton.destroy();
-}
-
-/** */
-function init() {
-    ExtensionUtils.initTranslations();
+    disable() {
+        Main.panel.menuManager.removeMenu(this._appsMenuButton.menu);
+        this._appsMenuButton.destroy();
+        delete this._appsMenuButton;
+    }
 }
