@@ -1,14 +1,22 @@
+// SPDX-FileCopyrightText: 2012 Giovanni Campagna <gcampagna@src.gnome.org>
+// SPDX-FileCopyrightText: 2013 Debarshi Ray <debarshir@gnome.org>
+// SPDX-FileCopyrightText: 2015 Florian Müllner <fmuellner@gnome.org>
+// SPDX-FileCopyrightText: 2016 Rémy Lefevre <lefevreremy@gmail.com>
+// SPDX-FileCopyrightText: 2017 Christian Kellner <christian@kellner.me>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 // -*- mode: js; js-indent-level: 4; indent-tabs-mode: nil -*-
-/* exported PlacesManager */
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import Shell from 'gi://Shell';
+import {EventEmitter} from 'resource:///org/gnome/shell/misc/signals.js';
 
-const {Gio, GLib, Shell} = imports.gi;
-const {EventEmitter} = imports.misc.signals;
+import {gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const ShellMountOperation = imports.ui.shellMountOperation;
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as ShellMountOperation from 'resource:///org/gnome/shell/ui/shellMountOperation.js';
 
-const _ = ExtensionUtils.gettext;
 const N_ = x => x;
 
 Gio._promisify(Gio.AppInfo, 'launch_default_for_uri_async');
@@ -248,7 +256,7 @@ const DEFAULT_DIRECTORIES = [
     GLib.UserDirectory.DIRECTORY_VIDEOS,
 ];
 
-var PlacesManager = class extends EventEmitter {
+export class PlacesManager extends EventEmitter {
     constructor() {
         super();
 
@@ -260,15 +268,25 @@ var PlacesManager = class extends EventEmitter {
         };
 
         this._settings = new Gio.Settings({schema_id: BACKGROUND_SCHEMA});
-        this._showDesktopIconsChangedId = this._settings.connect(
-            'changed::show-desktop-icons', this._updateSpecials.bind(this));
+        this._settings.connectObject('changed::show-desktop-icons',
+            () => this._updateSpecials(), this);
         this._updateSpecials();
 
         /*
         * Show devices, code more or less ported from nautilus-places-sidebar.c
         */
         this._volumeMonitor = Gio.VolumeMonitor.get();
-        this._connectVolumeMonitorSignals();
+        this._volumeMonitor.connectObject(
+            'volume-added', () => this._updateMounts(),
+            'volume-removed', () => this._updateMounts(),
+            'volume-changed', () => this._updateMounts(),
+            'mount-added', () => this._updateMounts(),
+            'mount-removed', () => this._updateMounts(),
+            'mount-changed', () => this._updateMounts(),
+            'drive-connected', () => this._updateMounts(),
+            'drive-disconnected', () => this._updateMounts(),
+            'drive-changed', () => this._updateMounts(),
+            this);
         this._updateMounts();
 
         this._bookmarksFile = this._findBookmarksFile();
@@ -293,34 +311,11 @@ var PlacesManager = class extends EventEmitter {
         }
     }
 
-    _connectVolumeMonitorSignals() {
-        const signals = [
-            'volume-added',
-            'volume-removed',
-            'volume-changed',
-            'mount-added',
-            'mount-removed',
-            'mount-changed',
-            'drive-connected',
-            'drive-disconnected',
-            'drive-changed',
-        ];
-
-        this._volumeMonitorSignals = [];
-        let func = this._updateMounts.bind(this);
-        for (let i = 0; i < signals.length; i++) {
-            let id = this._volumeMonitor.connect(signals[i], func);
-            this._volumeMonitorSignals.push(id);
-        }
-    }
-
     destroy() {
-        if (this._settings)
-            this._settings.disconnect(this._showDesktopIconsChangedId);
+        this._settings?.disconnectObject(this);
         this._settings = null;
 
-        for (let i = 0; i < this._volumeMonitorSignals.length; i++)
-            this._volumeMonitor.disconnect(this._volumeMonitorSignals[i]);
+        this._volumeMonitor.disconnectObject(this);
 
         if (this._monitor)
             this._monitor.cancel();
@@ -546,4 +541,4 @@ var PlacesManager = class extends EventEmitter {
     get(kind) {
         return this._places[kind];
     }
-};
+}
