@@ -1,21 +1,34 @@
-/* exported init */
-const {Clutter, Gio, GLib, GObject, Gtk, Meta, Shell, St} = imports.gi;
+// SPDX-FileCopyrightText: 2012 Florian Müllner <fmuellner@gnome.org>
+// SPDX-FileCopyrightText: 2013 Giovanni Campagna <gcampagna@src.gnome.org>
+// SPDX-FileCopyrightText: 2014 Sylvain Pasche <sylvain.pasche@gmail.com>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
 
-const DND = imports.ui.dnd;
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const Overview = imports.ui.overview;
-const PopupMenu = imports.ui.popupMenu;
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GObject from 'gi://GObject';
+import Gtk from 'gi://Gtk';
+import Meta from 'gi://Meta';
+import Shell from 'gi://Shell';
+import St from 'gi://St';
 
-const Me = ExtensionUtils.getCurrentExtension();
-const {WindowPicker, WindowPickerToggle} = Me.imports.windowPicker;
-const {WorkspaceIndicator} = Me.imports.workspaceIndicator;
+import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
-const _ = ExtensionUtils.gettext;
+import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as Overview from 'resource:///org/gnome/shell/ui/overview.js';
+import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
+
+import {WindowPicker, WindowPickerToggle} from './windowPicker.js';
+import {WorkspaceIndicator} from './workspaceIndicator.js';
 
 const ICON_TEXTURE_SIZE = 24;
 const DND_ACTIVATE_TIMEOUT = 500;
 
+const TOOLTIP_OFFSET = 6;
+const TOOLTIP_ANIMATION_TIME = 150;
+
 const GroupingMode = {
     NEVER: 0,
     AUTO: 1,
@@ -48,10 +61,6 @@ class WindowContextMenu extends PopupMenu.PopupMenu {
         });
         this.addMenuItem(this._minimizeItem);
 
-        this._notifyMinimizedId = this._metaWindow.connect(
-            'notify::minimized', this._updateMinimizeItem.bind(this));
-        this._updateMinimizeItem();
-
         this._maximizeItem = new PopupMenu.PopupMenuItem('');
         this._maximizeItem.connect('activate', () => {
             if (this._metaWindow.get_maximized() === Meta.MaximizeFlags.BOTH)
@@ -61,21 +70,20 @@ class WindowContextMenu extends PopupMenu.PopupMenu {
         });
         this.addMenuItem(this._maximizeItem);
 
-        this._notifyMaximizedHId = this._metaWindow.connect(
-            'notify::maximized-horizontally',
-            this._updateMaximizeItem.bind(this));
-        this._notifyMaximizedVId = this._metaWindow.connect(
-            'notify::maximized-vertically',
-            this._updateMaximizeItem.bind(this));
-        this._updateMaximizeItem();
-
         this._closeItem = new PopupMenu.PopupMenuItem(_('Close'));
         this._closeItem.connect('activate', () => {
             this._metaWindow.delete(global.get_current_time());
         });
         this.addMenuItem(this._closeItem);
 
-        this.actor.connect('destroy', this._onDestroy.bind(this));
+        this._metaWindow.connectObject(
+            'notify::minimized', this._updateMinimizeItem.bind(this),
+            'notify::maximized-horizontally', this._updateMaximizeItem.bind(this),
+            'notify::maximized-vertically', this._updateMaximizeItem.bind(this),
+            this);
+
+        this._updateMinimizeItem();
+        this._updateMaximizeItem();
 
         this.connect('open-state-changed', () => {
             if (!this.isOpen)
@@ -98,12 +106,6 @@ class WindowContextMenu extends PopupMenu.PopupMenu {
         this._maximizeItem.label.text = maximized
             ? _('Unmaximize') : _('Maximize');
     }
-
-    _onDestroy() {
-        this._metaWindow.disconnect(this._notifyMinimizedId);
-        this._metaWindow.disconnect(this._notifyMaximizedHId);
-        this._metaWindow.disconnect(this._notifyMaximizedVId);
-    }
 }
 
 class WindowTitle extends St.BoxLayout {
@@ -127,20 +129,19 @@ class WindowTitle extends St.BoxLayout {
         this.add(this.label_actor);
 
         this._textureCache = St.TextureCache.get_default();
-        this._iconThemeChangedId = this._textureCache.connect(
-            'icon-theme-changed', this._updateIcon.bind(this));
-        this._notifyWmClass = this._metaWindow.connect_after(
-            'notify::wm-class', this._updateIcon.bind(this));
-        this._notifyAppId = this._metaWindow.connect_after(
-            'notify::gtk-application-id', this._updateIcon.bind(this));
+        this._textureCache.connectObject('icon-theme-changed',
+            () => this._updateIcon(), this);
+
+        this._metaWindow.connectObject(
+            'notify::wm-class',
+            () => this._updateIcon(), GObject.ConnectFlags.AFTER,
+            'notify::gtk-application-id',
+            () => this._updateIcon(), GObject.ConnectFlags.AFTER,
+            'notify::title', () => this._updateTitle(),
+            'notify::minimized', () => this._minimizedChanged(),
+            this);
+
         this._updateIcon();
-
-        this.connect('destroy', this._onDestroy.bind(this));
-
-        this._notifyTitleId = this._metaWindow.connect(
-            'notify::title', this._updateTitle.bind(this));
-        this._notifyMinimizedId = this._metaWindow.connect(
-            'notify::minimized', this._minimizedChanged.bind(this));
         this._minimizedChanged();
     }
 
@@ -170,14 +171,6 @@ class WindowTitle extends St.BoxLayout {
             });
         }
     }
-
-    _onDestroy() {
-        this._textureCache.disconnect(this._iconThemeChangedId);
-        this._metaWindow.disconnect(this._notifyTitleId);
-        this._metaWindow.disconnect(this._notifyMinimizedId);
-        this._metaWindow.disconnect(this._notifyWmClass);
-        this._metaWindow.disconnect(this._notifyAppId);
-    }
 }
 
 class BaseButton extends St.Button {
@@ -213,17 +206,23 @@ class BaseButton extends St.Button {
 
         this._contextMenuManager = new PopupMenu.PopupMenuManager(this);
 
-        this._switchWorkspaceId = global.window_manager.connect(
-            'switch-workspace', this._updateVisibility.bind(this));
+        global.window_manager.connectObject('switch-workspace',
+            () => this._updateVisibility(), this);
 
         if (this._perMonitor) {
-            this._windowEnteredMonitorId = global.display.connect(
+            global.display.connectObject(
                 'window-entered-monitor',
-                this._windowEnteredOrLeftMonitor.bind(this));
-            this._windowLeftMonitorId = global.display.connect(
+                this._windowEnteredOrLeftMonitor.bind(this),
                 'window-left-monitor',
-                this._windowEnteredOrLeftMonitor.bind(this));
+                this._windowEnteredOrLeftMonitor.bind(this),
+                this);
         }
+
+        this._tooltip = new Tooltip(this, {
+            style_class: 'dash-label',
+            visible: false,
+        });
+        Main.uiGroup.add_child(this._tooltip);
     }
 
     get active() {
@@ -325,9 +324,11 @@ class BaseButton extends St.Button {
         if (isOpen)
             return;
 
+        const extension = Extension.lookupByURL(import.meta.url);
+
         let [x, y] = global.get_pointer();
         let actor = global.stage.get_actor_at_pos(Clutter.PickMode.REACTIVE, x, y);
-        if (Me.stateObj.someWindowListContains(actor))
+        if (extension.someWindowListContains(actor))
             actor.sync_hover();
     }
 
@@ -382,15 +383,7 @@ class BaseButton extends St.Button {
     }
 
     _onDestroy() {
-        global.window_manager.disconnect(this._switchWorkspaceId);
-
-        if (this._windowEnteredMonitorId)
-            global.display.disconnect(this._windowEnteredMonitorId);
-        this._windowEnteredMonitorId = 0;
-
-        if (this._windowLeftMonitorId)
-            global.display.disconnect(this._windowLeftMonitorId);
-        this._windowLeftMonitorId = 0;
+        this._tooltip.destroy();
     }
 }
 
@@ -403,9 +396,11 @@ class WindowButton extends BaseButton {
         super(perMonitor, monitorIndex);
 
         this.metaWindow = metaWindow;
-        this._skipTaskbarId = metaWindow.connect('notify::skip-taskbar', () => {
-            this._updateVisibility();
-        });
+        metaWindow.connectObject(
+            'notify::skip-taskbar', () => this._updateVisibility(),
+            'workspace-changed', () => this._updateVisibility(),
+            this);
+
         this._updateVisibility();
 
         this._windowTitle = new WindowTitle(this.metaWindow);
@@ -419,11 +414,8 @@ class WindowButton extends BaseButton {
         this._contextMenuManager.addMenu(this._contextMenu);
         Main.uiGroup.add_actor(this._contextMenu.actor);
 
-        this._workspaceChangedId = this.metaWindow.connect(
-            'workspace-changed', this._updateVisibility.bind(this));
-
-        this._notifyFocusId = global.display.connect(
-            'notify::focus-window', this._updateStyle.bind(this));
+        global.display.connectObject('notify::focus-window',
+            () => this._updateStyle(), this);
         this._updateStyle();
     }
 
@@ -467,9 +459,6 @@ class WindowButton extends BaseButton {
 
     _onDestroy() {
         super._onDestroy();
-        this.metaWindow.disconnect(this._skipTaskbarId);
-        this.metaWindow.disconnect(this._workspaceChangedId);
-        global.display.disconnect(this._notifyFocusId);
         this._contextMenu.destroy();
     }
 }
@@ -586,18 +575,17 @@ class AppButton extends BaseButton {
         Main.uiGroup.add_actor(this._appContextMenu.actor);
 
         this._textureCache = St.TextureCache.get_default();
-        this._iconThemeChangedId =
-            this._textureCache.connect('icon-theme-changed', () => {
-                this._icon.child = app.create_icon_texture(ICON_TEXTURE_SIZE);
-            });
+        this._textureCache.connectObject('icon-theme-changed', () => {
+            this._icon.child = app.create_icon_texture(ICON_TEXTURE_SIZE);
+        }, this);
 
-        this._windowsChangedId = this.app.connect(
-            'windows-changed', this._windowsChanged.bind(this));
+        this.app.connectObject('windows-changed',
+            () => this._windowsChanged(), this);
         this._windowsChanged();
 
         this._windowTracker = Shell.WindowTracker.get_default();
-        this._notifyFocusId = this._windowTracker.connect(
-            'notify::focus-app', this._updateStyle.bind(this));
+        this._windowTracker.connectObject('notify::focus-app',
+            () => this._updateStyle(), this);
         this._updateStyle();
     }
 
@@ -717,9 +705,6 @@ class AppButton extends BaseButton {
 
     _onDestroy() {
         super._onDestroy();
-        this._textureCache.disconnect(this._iconThemeChangedId);
-        this._windowTracker.disconnect(this._notifyFocusId);
-        this.app.disconnect(this._windowsChangedId);
         this._menu.destroy();
     }
 }
@@ -729,7 +714,7 @@ class WindowList extends St.Widget {
         GObject.registerClass(this);
     }
 
-    constructor(perMonitor, monitor) {
+    constructor(perMonitor, monitor, settings) {
         super({
             name: 'panel',
             style_class: 'bottom-panel solid',
@@ -776,12 +761,12 @@ class WindowList extends St.Widget {
         indicatorsBox.add_child(this._workspaceIndicator.container);
 
         this._mutterSettings = new Gio.Settings({schema_id: 'org.gnome.mutter'});
-        this._workspacesOnlyOnPrimaryChangedId = this._mutterSettings.connect(
+        this._mutterSettings.connectObject(
             'changed::workspaces-only-on-primary',
-            this._updateWorkspaceIndicatorVisibility.bind(this));
-        this._dynamicWorkspacesChangedId = this._mutterSettings.connect(
+            () => this._updateWorkspaceIndicatorVisibility(),
             'changed::dynamic-workspaces',
-            this._updateWorkspaceIndicatorVisibility.bind(this));
+            () => this._updateWorkspaceIndicatorVisibility(),
+            this);
         this._updateWorkspaceIndicatorVisibility();
 
         this._menuManager = new PopupMenu.PopupMenuManager(this);
@@ -799,59 +784,58 @@ class WindowList extends St.Widget {
         this._updatePosition();
 
         this._appSystem = Shell.AppSystem.get_default();
-        this._appStateChangedId = this._appSystem.connect(
-            'app-state-changed', this._onAppStateChanged.bind(this));
+        this._appSystem.connectObject('app-state-changed',
+            this._onAppStateChanged.bind(this), this);
 
         // Hack: OSK gesture is tied to visibility, piggy-back on that
-        this._keyboardVisiblechangedId =
-            Main.keyboard._bottomDragAction.connect('notify::enabled',
-                action => {
-                    const visible = !action.enabled;
-                    if (visible) {
-                        Main.uiGroup.set_child_above_sibling(
-                            this, Main.layoutManager.keyboardBox);
-                    } else {
-                        Main.uiGroup.set_child_above_sibling(
-                            this, Main.layoutManager.panelBox);
-                    }
-                    this._updateKeyboardAnchor();
-                });
+        Main.keyboard._bottomDragAction.connectObject('notify::enabled',
+            action => {
+                const visible = !action.enabled;
+                if (visible) {
+                    Main.uiGroup.set_child_above_sibling(
+                        this, Main.layoutManager.keyboardBox);
+                } else {
+                    Main.uiGroup.set_child_above_sibling(
+                        this, Main.layoutManager.panelBox);
+                }
+                this._updateKeyboardAnchor();
+            }, this);
 
         let workspaceManager = global.workspace_manager;
 
-        this._nWorkspacesChangedId = workspaceManager.connect(
-            'notify::n-workspaces', this._updateWorkspaceIndicatorVisibility.bind(this));
+        workspaceManager.connectObject('notify::n-workspaces',
+            () => this._updateWorkspaceIndicatorVisibility(), this);
         this._updateWorkspaceIndicatorVisibility();
 
-        this._switchWorkspaceId = global.window_manager.connect(
-            'switch-workspace', this._checkGrouping.bind(this));
+        global.window_manager.connectObject('switch-workspace',
+            () => this._checkGrouping(), this);
 
-        this._overviewShowingId = Main.overview.connect('showing', () => {
-            this.hide();
-            this._updateKeyboardAnchor();
-        });
-
-        this._overviewHidingId = Main.overview.connect('hidden', () => {
-            this.visible = !this._monitor.inFullscreen;
-            this._updateKeyboardAnchor();
-        });
-
-        this._fullscreenChangedId =
-            global.display.connect('in-fullscreen-changed', () => {
-                // Work-around for initial change from unknown to !fullscreen
-                if (Main.overview.visible)
-                    this.hide();
+        Main.overview.connectObject(
+            'showing', () => {
+                this.hide();
                 this._updateKeyboardAnchor();
-            });
+            },
+            'hidden', () => {
+                this.visible = !this._monitor.inFullscreen;
+                this._updateKeyboardAnchor();
+            }, this);
+
+        global.display.connectObject('in-fullscreen-changed', () => {
+            // Work-around for initial change from unknown to !fullscreen
+            if (Main.overview.visible)
+                this.hide();
+            this._updateKeyboardAnchor();
+        }, this);
 
         this._windowSignals = new Map();
         this._windowCreatedId = global.display.connect(
             'window-created', (dsp, win) => this._addWindow(win));
 
-        this._dragBeginId = Main.xdndHandler.connect('drag-begin',
-            this._monitorDrag.bind(this));
-        this._dragEndId = Main.xdndHandler.connect('drag-end',
-            this._stopMonitoringDrag.bind(this));
+        Main.xdndHandler.connectObject(
+            'drag-begin', () => this._monitorDrag(),
+            'drag-end', () => this._stopMonitoringDrag(),
+            this);
+
         this._dragMonitor = {
             dragMotion: this._onDragMotion.bind(this),
         };
@@ -859,9 +843,9 @@ class WindowList extends St.Widget {
         this._dndTimeoutId = 0;
         this._dndWindow = null;
 
-        this._settings = ExtensionUtils.getSettings();
-        this._groupingModeChangedId = this._settings.connect(
-            'changed::grouping-mode', this._groupingModeChanged.bind(this));
+        this._settings = settings;
+        this._settings.connect('changed::grouping-mode',
+            () => this._groupingModeChanged());
         this._grouped = undefined;
         this._groupingModeChanged();
     }
@@ -900,7 +884,8 @@ class WindowList extends St.Widget {
     }
 
     _updateWindowListVisibility() {
-        let visible = !Main.windowPicker.visible;
+        const {windowPicker} = Extension.lookupByURL(import.meta.url);
+        const visible = !windowPicker.visible;
 
         this._windowList.ease({
             opacity: visible ? 255 : 0,
@@ -1098,39 +1083,16 @@ class WindowList extends St.Widget {
     }
 
     _onDestroy() {
-        this._mutterSettings.disconnect(this._workspacesOnlyOnPrimaryChangedId);
-        this._mutterSettings.disconnect(this._dynamicWorkspacesChangedId);
-
         this._workspaceIndicator.destroy();
 
         Main.ctrlAltTabManager.removeGroup(this);
 
-        this._appSystem.disconnect(this._appStateChangedId);
-        this._appStateChangedId = 0;
-
-        Main.keyboard._bottomDragAction.disconnect(this._keyboardVisiblechangedId);
-        this._keyboardVisiblechangedId = 0;
-
-        global.workspace_manager.disconnect(this._nWorkspacesChangedId);
-        this._nWorkspacesChangedId = 0;
-
-        global.window_manager.disconnect(this._switchWorkspaceId);
-        this._switchWorkspaceId = 0;
-
         this._windowSignals.forEach((id, win) => win.disconnect(id));
         this._windowSignals.clear();
 
-        Main.overview.disconnect(this._overviewShowingId);
-        Main.overview.disconnect(this._overviewHidingId);
-
-        global.display.disconnect(this._fullscreenChangedId);
-        global.display.disconnect(this._windowCreatedId);
-
         this._stopMonitoringDrag();
-        Main.xdndHandler.disconnect(this._dragBeginId);
-        Main.xdndHandler.disconnect(this._dragEndId);
 
-        this._settings.disconnect(this._groupingModeChangedId);
+        this._settings.run_dispose();
 
         let windows = global.get_window_actors();
         for (let i = 0; i < windows.length; i++)
@@ -1138,9 +1100,9 @@ class WindowList extends St.Widget {
     }
 }
 
-class Extension {
-    constructor() {
-        ExtensionUtils.initTranslations();
+export default class WindowListExtension extends Extension {
+    constructor(metadata) {
+        super(metadata);
 
         this._windowLists = null;
         this._hideOverviewOrig = Main.overview.hide;
@@ -1149,17 +1111,17 @@ class Extension {
     enable() {
         this._windowLists = [];
 
-        this._settings = ExtensionUtils.getSettings();
-        this._showOnAllMonitorsChangedId = this._settings.connect(
-            'changed::show-on-all-monitors', this._buildWindowLists.bind(this));
+        this._settings = this.getSettings();
+        this._settings.connectObject('changed::show-on-all-monitors',
+            () => this._buildWindowLists(), this);
 
-        this._monitorsChangedId = Main.layoutManager.connect(
-            'monitors-changed', this._buildWindowLists.bind(this));
+        Main.layoutManager.connectObject('monitors-changed',
+            () => this._buildWindowLists(), this);
 
-        Main.windowPicker = new WindowPicker();
+        this.windowPicker = new WindowPicker();
 
         Main.overview.hide = () => {
-            Main.windowPicker.close();
+            this.windowPicker.close();
             this._hideOverviewOrig.call(Main.overview);
         };
 
@@ -1174,7 +1136,7 @@ class Extension {
 
         Main.layoutManager.monitors.forEach(monitor => {
             if (showOnAllMonitors || monitor === Main.layoutManager.primaryMonitor)
-                this._windowLists.push(new WindowList(showOnAllMonitors, monitor));
+                this._windowLists.push(new WindowList(showOnAllMonitors, monitor, this.getSettings()));
         });
     }
 
@@ -1182,11 +1144,8 @@ class Extension {
         if (!this._windowLists)
             return;
 
-        this._settings.disconnect(this._showOnAllMonitorsChangedId);
-        this._showOnAllMonitorsChangedId = 0;
-
-        Main.layoutManager.disconnect(this._monitorsChangedId);
-        this._monitorsChangedId = 0;
+        this._settings.disconnectObject(this);
+        Main.layoutManager.disconnectObject(this);
 
         this._windowLists.forEach(windowList => {
             windowList.hide();
@@ -1194,8 +1153,8 @@ class Extension {
         });
         this._windowLists = null;
 
-        Main.windowPicker.destroy();
-        delete Main.windowPicker;
+        this.windowPicker.destroy();
+        delete this.windowPicker;
 
         Main.overview.hide = this._hideOverviewOrig;
     }
@@ -1205,9 +1164,66 @@ class Extension {
     }
 }
 
-/**
- * @returns {Extension} - the extension's state object
- */
-function init() {
-    return new Extension();
+class Tooltip extends St.Label {
+    static {
+        GObject.registerClass(this);
+    }
+
+    constructor(widget, params) {
+        super(params);
+
+        this._widget = widget;
+
+        this._widget.connect('notify::hover', () => {
+            if (this._widget.hover)
+                this.open();
+            else
+                this.close();
+        });
+    }
+
+    open() {
+        const buttonTitleWidget = this._widget.label_actor;
+        const [, , preferredTitleWidth] = buttonTitleWidget.get_preferred_size();
+        const maxTitleWidth = buttonTitleWidget.allocation.get_width();
+        const isTitleFullyShown = preferredTitleWidth <= maxTitleWidth;
+
+        if (isTitleFullyShown)
+            return;
+
+        this.set({
+            text: this._widget.label_actor.get_text(),
+            visible: true,
+            opacity: 0,
+        });
+
+        const [stageX, stageY] = this._widget.get_transformed_position();
+        const thumbWidth = this._widget.allocation.get_width();
+        const tipWidth = this.width;
+        const tipHeight = this.height;
+        const xOffset = Math.floor((thumbWidth - tipWidth) / 2);
+        const monitor = Main.layoutManager.findMonitorForActor(this);
+        const x = Math.clamp(
+            stageX + xOffset,
+            monitor.x,
+            monitor.x + monitor.width - tipWidth);
+        const y = stageY - tipHeight - TOOLTIP_OFFSET;
+        this.set_position(x, y);
+
+        this.ease({
+            opacity: 255,
+            duration: TOOLTIP_ANIMATION_TIME,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => (this.visible = this._widget.hover),
+        });
+    }
+
+    close() {
+        this.ease({
+            opacity: 0,
+            duration: TOOLTIP_ANIMATION_TIME,
+            mode: Clutter.AnimationMode.EASE_OUT_QUAD,
+            onComplete: () => (this.visible = this._widget.hover),
+        });
+    }
 }
