@@ -1,13 +1,19 @@
-/* exported WorkspaceIndicator */
-const {Clutter, Gio, GObject, Meta, St} = imports.gi;
+// SPDX-FileCopyrightText: 2019 Florian Müllner <fmuellner@gnome.org>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
 
-const DND = imports.ui.dnd;
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const PanelMenu = imports.ui.panelMenu;
-const PopupMenu = imports.ui.popupMenu;
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GObject from 'gi://GObject';
+import Meta from 'gi://Meta';
+import St from 'gi://St';
 
-const _ = ExtensionUtils.gettext;
+import {gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
+
+import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
+import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
 
 const TOOLTIP_OFFSET = 6;
 const TOOLTIP_ANIMATION_TIME = 150;
@@ -29,20 +35,17 @@ class WindowPreview extends St.Button {
 
         this._window = window;
 
-        this.connect('destroy', this._onDestroy.bind(this));
-
-        this._sizeChangedId = this._window.connect('size-changed',
-            () => this.queue_relayout());
-        this._positionChangedId = this._window.connect('position-changed',
-            () => {
+        this._window.connectObject(
+            'size-changed', () => this.queue_relayout(),
+            'position-changed', () => {
                 this._updateVisible();
                 this.queue_relayout();
-            });
-        this._minimizedChangedId = this._window.connect('notify::minimized',
-            this._updateVisible.bind(this));
+            },
+            'notify::minimized', this._updateVisible.bind(this),
+            this);
 
-        this._focusChangedId = global.display.connect('notify::focus-window',
-            this._onFocusChanged.bind(this));
+        global.display.connectObject('notify::focus-window',
+            this._onFocusChanged.bind(this), this);
         this._onFocusChanged();
     }
 
@@ -51,13 +54,6 @@ class WindowPreview extends St.Button {
         return this._window;
     }
 
-    _onDestroy() {
-        this._window.disconnect(this._sizeChangedId);
-        this._window.disconnect(this._positionChangedId);
-        this._window.disconnect(this._minimizedChangedId);
-        global.display.disconnect(this._focusChangedId);
-    }
-
     _onFocusChanged() {
         if (global.display.focus_window === this._window)
             this.add_style_class_name('active');
@@ -138,16 +134,13 @@ class WorkspaceThumbnail extends St.Button {
         let workspaceManager = global.workspace_manager;
         this._workspace = workspaceManager.get_workspace_by_index(index);
 
-        this._windowAddedId = this._workspace.connect('window-added',
-            (ws, window) => {
-                this._addWindow(window);
-            });
-        this._windowRemovedId = this._workspace.connect('window-removed',
-            (ws, window) => {
-                this._removeWindow(window);
-            });
-        this._restackedId = global.display.connect('restacked',
-            this._onRestacked.bind(this));
+        this._workspace.connectObject(
+            'window-added', (ws, window) => this._addWindow(window),
+            'window-removed', (ws, window) => this._removeWindow(window),
+            this);
+
+        global.display.connectObject('restacked',
+            this._onRestacked.bind(this), this);
 
         this._workspace.list_windows().forEach(w => this._addWindow(w));
         this._onRestacked();
@@ -245,20 +238,16 @@ class WorkspaceThumbnail extends St.Button {
 
     _onDestroy() {
         this._tooltip.destroy();
-
-        this._workspace.disconnect(this._windowAddedId);
-        this._workspace.disconnect(this._windowRemovedId);
-        global.display.disconnect(this._restackedId);
     }
 }
 
-var WorkspaceIndicator = class WorkspaceIndicator extends PanelMenu.Button {
+export class WorkspaceIndicator extends PanelMenu.Button {
     static {
         GObject.registerClass(this);
     }
 
     constructor() {
-        super(0.0, _('Workspace Indicator'), true);
+        super(0.5, _('Workspace Indicator'), true);
         this.setMenu(new PopupMenu.PopupMenu(this, 0.0, St.Side.BOTTOM));
         this.add_style_class_name('window-list-workspace-indicator');
         this.remove_style_class_name('panel-button');
@@ -295,14 +284,11 @@ var WorkspaceIndicator = class WorkspaceIndicator extends PanelMenu.Button {
 
         this._workspacesItems = [];
 
-        this._workspaceManagerSignals = [
-            workspaceManager.connect('notify::n-workspaces',
-                this._nWorkspacesChanged.bind(this)),
-            workspaceManager.connect_after('workspace-switched',
-                this._onWorkspaceSwitched.bind(this)),
-            workspaceManager.connect('notify::layout-rows',
-                this._updateThumbnailVisibility.bind(this)),
-        ];
+        workspaceManager.connectObject(
+            'notify::n-workspaces', this._nWorkspacesChanged.bind(this), GObject.ConnectFlags.AFTER,
+            'workspace-switched', this._onWorkspaceSwitched.bind(this), GObject.ConnectFlags.AFTER,
+            'notify::layout-rows', this._updateThumbnailVisibility.bind(this),
+            this);
 
         this.connect('scroll-event', this._onScrollEvent.bind(this));
         this._updateMenu();
@@ -310,20 +296,8 @@ var WorkspaceIndicator = class WorkspaceIndicator extends PanelMenu.Button {
         this._updateThumbnailVisibility();
 
         this._settings = new Gio.Settings({schema_id: 'org.gnome.desktop.wm.preferences'});
-        this._settingsChangedId = this._settings.connect(
-            'changed::workspace-names', this._updateMenuLabels.bind(this));
-    }
-
-    _onDestroy() {
-        for (let i = 0; i < this._workspaceManagerSignals.length; i++)
-            global.workspace_manager.disconnect(this._workspaceManagerSignals[i]);
-
-        if (this._settingsChangedId) {
-            this._settings.disconnect(this._settingsChangedId);
-            this._settingsChangedId = 0;
-        }
-
-        super._onDestroy();
+        this._settings.connectObject('changed::workspace-names',
+            () => this._updateMenuLabels(), this);
     }
 
     _updateThumbnailVisibility() {
@@ -447,4 +421,4 @@ var WorkspaceIndicator = class WorkspaceIndicator extends PanelMenu.Button {
         let newIndex = this._currentWorkspace + diff;
         this._activate(newIndex);
     }
-};
+}
