@@ -1,15 +1,22 @@
+// SPDX-FileCopyrightText: 2011 Erick Pérez Castellanos <erick.red@gmail.com>
+// SPDX-FileCopyrightText: 2011 Giovanni Campagna <gcampagna@src.gnome.org>
+// SPDX-FileCopyrightText: 2017 Florian Müllner <fmuellner@gnome.org>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
+
 // -*- mode: js2; indent-tabs-mode: nil; js2-basic-offset: 4 -*-
-/* exported init enable disable */
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GObject from 'gi://GObject';
+import Meta from 'gi://Meta';
+import St from 'gi://St';
 
-const {Clutter, Gio, GObject, Meta, St} = imports.gi;
+import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
-const DND = imports.ui.dnd;
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const PanelMenu = imports.ui.panelMenu;
-const PopupMenu = imports.ui.popupMenu;
-
-const _ = ExtensionUtils.gettext;
+import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
+import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
 
 const WORKSPACE_SCHEMA = 'org.gnome.desktop.wm.preferences';
 const WORKSPACE_KEY = 'workspace-names';
@@ -34,20 +41,17 @@ class WindowPreview extends St.Button {
 
         this._window = window;
 
-        this.connect('destroy', this._onDestroy.bind(this));
-
-        this._sizeChangedId = this._window.connect('size-changed',
-            () => this.queue_relayout());
-        this._positionChangedId = this._window.connect('position-changed',
-            () => {
+        this._window.connectObject(
+            'size-changed', () => this.queue_relayout(),
+            'position-changed', () => {
                 this._updateVisible();
                 this.queue_relayout();
-            });
-        this._minimizedChangedId = this._window.connect('notify::minimized',
-            this._updateVisible.bind(this));
+            },
+            'notify::minimized', this._updateVisible.bind(this),
+            this);
 
-        this._focusChangedId = global.display.connect('notify::focus-window',
-            this._onFocusChanged.bind(this));
+        global.display.connectObject('notify::focus-window',
+            this._onFocusChanged.bind(this), this);
         this._onFocusChanged();
     }
 
@@ -56,13 +60,6 @@ class WindowPreview extends St.Button {
         return this._window;
     }
 
-    _onDestroy() {
-        this._window.disconnect(this._sizeChangedId);
-        this._window.disconnect(this._positionChangedId);
-        this._window.disconnect(this._minimizedChangedId);
-        global.display.disconnect(this._focusChangedId);
-    }
-
     _onFocusChanged() {
         if (global.display.focus_window === this._window)
             this.add_style_class_name('active');
@@ -143,16 +140,13 @@ class WorkspaceThumbnail extends St.Button {
         let workspaceManager = global.workspace_manager;
         this._workspace = workspaceManager.get_workspace_by_index(index);
 
-        this._windowAddedId = this._workspace.connect('window-added',
-            (ws, window) => {
-                this._addWindow(window);
-            });
-        this._windowRemovedId = this._workspace.connect('window-removed',
-            (ws, window) => {
-                this._removeWindow(window);
-            });
-        this._restackedId = global.display.connect('restacked',
-            this._onRestacked.bind(this));
+        this._workspace.connectObject(
+            'window-added', (ws, window) => this._addWindow(window),
+            'window-removed', (ws, window) => this._removeWindow(window),
+            this);
+
+        global.display.connectObject('restacked',
+            this._onRestacked.bind(this), this);
 
         this._workspace.list_windows().forEach(w => this._addWindow(w));
         this._onRestacked();
@@ -250,10 +244,6 @@ class WorkspaceThumbnail extends St.Button {
 
     _onDestroy() {
         this._tooltip.destroy();
-
-        this._workspace.disconnect(this._windowAddedId);
-        this._workspace.disconnect(this._windowRemovedId);
-        global.display.disconnect(this._restackedId);
     }
 }
 
@@ -295,14 +285,11 @@ class WorkspaceIndicator extends PanelMenu.Button {
         this._workspaceSection = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(this._workspaceSection);
 
-        this._workspaceManagerSignals = [
-            workspaceManager.connect_after('notify::n-workspaces',
-                this._nWorkspacesChanged.bind(this)),
-            workspaceManager.connect_after('workspace-switched',
-                this._onWorkspaceSwitched.bind(this)),
-            workspaceManager.connect('notify::layout-rows',
-                this._updateThumbnailVisibility.bind(this)),
-        ];
+        workspaceManager.connectObject(
+            'notify::n-workspaces', this._nWorkspacesChanged.bind(this), GObject.ConnectFlags.AFTER,
+            'workspace-switched', this._onWorkspaceSwitched.bind(this), GObject.ConnectFlags.AFTER,
+            'notify::layout-rows', this._updateThumbnailVisibility.bind(this),
+            this);
 
         this.connect('scroll-event', this._onScrollEvent.bind(this));
         this._thumbnailsBox.connect('scroll-event', this._onScrollEvent.bind(this));
@@ -311,20 +298,11 @@ class WorkspaceIndicator extends PanelMenu.Button {
         this._updateThumbnailVisibility();
 
         this._settings = new Gio.Settings({schema_id: WORKSPACE_SCHEMA});
-        this._settingsChangedId = this._settings.connect(
-            `changed::${WORKSPACE_KEY}`,
-            this._updateMenuLabels.bind(this));
+        this._settings.connectObject(`changed::${WORKSPACE_KEY}`,
+            this._updateMenuLabels.bind(this), this);
     }
 
     _onDestroy() {
-        for (let i = 0; i < this._workspaceManagerSignals.length; i++)
-            global.workspace_manager.disconnect(this._workspaceManagerSignals[i]);
-
-        if (this._settingsChangedId) {
-            this._settings.disconnect(this._settingsChangedId);
-            this._settingsChangedId = 0;
-        }
-
         Main.panel.set_offscreen_redirect(Clutter.OffscreenRedirect.ALWAYS);
 
         super._onDestroy();
@@ -454,20 +432,14 @@ class WorkspaceIndicator extends PanelMenu.Button {
     }
 }
 
-/** */
-function init() {
-    ExtensionUtils.initTranslations();
-}
+export default class WorkspaceIndicatorExtension extends Extension {
+    enable() {
+        this._indicator = new WorkspaceIndicator();
+        Main.panel.addToStatusArea('workspace-indicator', this._indicator);
+    }
 
-let _indicator;
-
-/** */
-function enable() {
-    _indicator = new WorkspaceIndicator();
-    Main.panel.addToStatusArea('workspace-indicator', _indicator);
-}
-
-/** */
-function disable() {
-    _indicator.destroy();
+    disable() {
+        this._indicator.destroy();
+        delete this._indicator;
+    }
 }
